-- set targetless plugin lock states
--
-- > targetless.api.radarlock to prevent radar control interference
--
-- > targetless.var.lock to prevent targetless.roidtype from causing lag
function PR.Interop.SetTargetlessLock(state)
    if PR.Interop.IsTargetlessInstalled then
        targetless.api.radarlock = state
        targetless.var.lock = state
    end
end

function PR.Interop.ReleaseTargetlessLock()
    return PR.Interop.SetTargetlessLock(false)
end

-- load targetless roid data
local function loadTargetlessData()
    local success, notesid = pcall(function() return targetless.var.noteoffset end)
    if not success then notesid = 741937 end
    return unspickle(LoadSystemNotes(notesid))
end

local function saveTargetlessData(targetless_data)
    local success, notesid = pcall(function() return targetless.var.noteoffset end)
    if not success then notesid = 741937 end
    return SaveSystemNotes(spickle(targetless_data), notesid)
end

local function importTargetlessData(targetless_data)
    local imported = 0
    local skipped = 0
    local failed = 0
    for sectorId, targetless_sectorData in pairs(targetless_data) do
        -- load targetless data for one sector
        targetless_sectorData = unspickle(targetless_sectorData)
        if targetless_sectorData then
            -- load PR data for the same sector
            local sectorData = PR.LoadSectorData(sectorId)
            local sectorChanged = false
            if not sectorData.RoidList then sectorData.RoidList = {} end
            for roidId, targetlessRoidData in pairs(targetless_sectorData) do
                local roidData = sectorData.RoidList[roidId]
                if roidData then
                    -- skip asteroids we already know about
                    skipped = skipped + 1
                else
                    roidData = {}
                    for name, percentage in pairs(targetlessRoidData.ore) do
                        local shortname = PR.GetOreNameShort(name)
                        if shortname == name then
                            failed = failed + 1
                        else
                            roidData[shortname] = tonumber(percentage)
                        end
                    end
                    imported = imported + 1
                    sectorChanged = true
                end
                -- XXX: Consider GUI support for notes, then enable this
                --[[ -- import roid notes from targetless
                if not roidData.note and targetlessRoidData.note ~= "" then
                    roidData.note = targetlessRoidData.note
                    sectorChanged = true
                end ]]
                if sectorChanged then sectorData.RoidList[roidId] = roidData end
            end
            -- save PR sector data if something was imported
            if sectorChanged then
                PR.SaveSectorData(sectorId, sectorData)
                -- this causes a stack overflow for some reason, that's why
                -- we have to do the full PR.RecreateAllSystemData at the end
                --PR.SystemDataUpdate(sectorId)
            end
        end
    end
    return imported, skipped, failed
end

-- import new data from targetless db
function PR.Interop.ImportFromTargetless()
    PR.Interop.SetTargetlessLock(true)
    PR.Interop.Import("targetless", loadTargetlessData, importTargetlessData)
    PR.Interop.SetTargetlessLock(false)
end

-- adds PR sector data into targetless_data, returns count of added asteroids
local function mergeSectorIntoTargetlessData(sectorId, targetless_data)
    local export_count = 0
    local sectorData = PR.LoadSectorData(sectorId)
    if sectorData and type(sectorData) == "table" and
        sectorData.RoidList and next(sectorData.RoidList) then
        -- load targetless data for one sector
        local targetless_sectorData = unspickle(targetless_data[sectorId] or "")
        for roidId, roidTable in pairs(sectorData.RoidList) do
            if not targetless_sectorData[roidId] then
                targetless_sectorData[roidId] = {}
            end
            targetless_sectorData[roidId].id = roidId
            targetless_sectorData[roidId].note = roidTable.note or ""
            -- delete targetless roid data, use ours instead
            targetless_sectorData[roidId].ore = {}
            for i = 1, PR.Ore.n do
                local percentage = roidTable[PR.Ore[i].shortname] or 0
                if percentage > 0 then
                    targetless_sectorData[roidId].ore[PR.Ore[i].name] = tostring(percentage)
                end
            end
            export_count = export_count + 1
        end
        -- store updated sector data to targetless_data
        targetless_data[sectorId] = spickle(targetless_sectorData)
    end
    return export_count
end

-- export current sector data to Targetless db and refresh running Targetless
local function export_sector_to_targetless(targetless_data)
    PR.Print("Exporting asteroid data for current sector to Targetless, please wait...", true)
    local export_count = mergeSectorIntoTargetlessData(PR.Sector, targetless_data)
    return export_count
end

-- export all data to Targetless db and refresh running Targetless
local function export_all_to_targetless(targetless_data)
    PR.Print("Exporting all asteroid data to Targetless, please wait...", true)
    local export_count = 0
    for systemId = 1, PR.SystemOwnership.n do
        for sectorId = systemId * 256 - 255, systemId * 256 do
            export_count = export_count +
                mergeSectorIntoTargetlessData(sectorId, targetless_data)
        end
    end
    return export_count
end

function PR.Interop.ExportToTargetless(_, args)
    if not args or (args[1] ~= "sector" and args[1] ~= "all") then
        PR.Print([[
Export Prospector's asteroid data into the database used by the Targetless plugin.
In case of conflict, Prospector's data takes precedence. 
If Targetless is installed, it will immediately be refreshed with the new data.

Usage:
	/prExportToTargetless sector
		- exports the asteroid data for the current sector only
	/prExportToTargetless all
		- exports all asteroid data stored by Prospector
]]       , true)
    else
        PR.SaveNow()
        PR.Interop.SetTargetlessLock(true)
        -- load existing Targetless database, if any
        local targetless_data = loadTargetlessData()

        local export_count = 0
        -- merge data for curent or all sectors into targetless_data
        if args[1] == "sector" then
            export_count = export_sector_to_targetless(targetless_data)
        else
            export_count = export_all_to_targetless(targetless_data)
        end
        -- save targetless_data to Targetless database
        saveTargetlessData(targetless_data)
        -- update Targetless, if it's running
        if PR.IsTargetlessInstalled then
            pcall(function()
                targetless.RoidList.allroids = targetless_data
                targetless.RoidList:load(GetCurrentSectorid())
            end)
        end
        PR.Interop.SetTargetlessLock(false)
        if PR.IsTargetlessInstalled then
            pcall(function() targetless.Controller:update() end)
        end
        PR.Print("Export finished. " .. export_count ..
            " asteroids were exported to Targetless.", true)
    end
end
